@@ -466,12 +466,12 @@ _gdk_display_enable_motion_hints (GdkDisplay *display)
 /**
  * gdk_display_get_pointer:
  * @display: a #GdkDisplay
- * @screen: location to store the screen that the
+ * @screen: (allow-none): location to store the screen that the
  *          cursor is on, or %NULL.
- * @x: location to store root window X coordinate of pointer, or %NULL.
- * @y: location to store root window Y coordinate of pointer, or %NULL.
- * @mask: location to store current modifier mask, or %NULL
- * 
+ * @x: (out) (allow-none): location to store root window X coordinate of pointer, or %NULL.
+ * @y: (out) (allow-none): location to store root window Y coordinate of pointer, or %NULL.
+ * @mask: (out) (allow-none): location to store current modifier mask, or %NULL
+ *
  * Gets the current location of the pointer and the current modifier
  * mask for a given display.
  *
@@ -510,7 +510,7 @@ gdk_display_real_get_window_at_pointer (GdkDisplay *display,
   GdkWindow *window;
   gint x, y;
 
-  window = _gdk_windowing_window_at_pointer (display, &x, &y, NULL);
+  window = _gdk_windowing_window_at_pointer (display, &x, &y, NULL, FALSE);
 
   /* This might need corrections, as the native window returned
      may contain client side children */
@@ -567,17 +567,17 @@ gdk_window_real_window_get_pointer (GdkDisplay       *display,
 /**
  * gdk_display_get_window_at_pointer:
  * @display: a #GdkDisplay
- * @win_x: return location for x coordinate of the pointer location relative 
+ * @win_x: (out) (allow-none): return location for x coordinate of the pointer location relative
  *    to the window origin, or %NULL
- * @win_y: return location for y coordinate of the pointer location relative
+ * @win_y: (out) (allow-none): return location for y coordinate of the pointer location relative
  &    to the window origin, or %NULL
- * 
+ *
  * Obtains the window underneath the mouse pointer, returning the location
- * of the pointer in that window in @win_x, @win_y for @screen. Returns %NULL 
+ * of the pointer in that window in @win_x, @win_y for @screen. Returns %NULL
  * if the window under the mouse pointer is not known to GDK (for example, 
  * belongs to another application).
- * 
- * Returns: the window under the mouse pointer, or %NULL
+ *
+ * Returns: (transfer none): the window under the mouse pointer, or %NULL
  *
  * Since: 2.2
  **/
@@ -913,34 +913,20 @@ get_current_toplevel (GdkDisplay *display,
 		      GdkModifierType *state_out)
 {
   GdkWindow *pointer_window;
-  GdkWindowObject *w;
   int x, y;
   GdkModifierType state;
 
-  pointer_window = _gdk_windowing_window_at_pointer (display,  &x, &y, &state);
+  pointer_window = _gdk_windowing_window_at_pointer (display,  &x, &y, &state, TRUE);
   if (pointer_window != NULL &&
       (GDK_WINDOW_DESTROYED (pointer_window) ||
        GDK_WINDOW_TYPE (pointer_window) == GDK_WINDOW_ROOT ||
        GDK_WINDOW_TYPE (pointer_window) == GDK_WINDOW_FOREIGN))
     pointer_window = NULL;
 
-  w = (GdkWindowObject *)pointer_window;
-  if (w)
-    {
-      /* Convert to toplevel */
-      while (w->parent != NULL &&
-	     w->parent->window_type != GDK_WINDOW_ROOT)
-	{
-	  x += w->x;
-	  y += w->y;
-	  w = w->parent;
-	}
-    }
-
   *x_out = x;
   *y_out = y;
   *state_out = state;
-  return (GdkWindow *)w;
+  return pointer_window;
 }
 
 static void
@@ -1060,9 +1046,7 @@ _gdk_display_pointer_grab_update (GdkDisplay *display,
       if (current_grab->serial_start > current_serial)
 	return; /* Hasn't started yet */
       
-      if (current_grab->serial_end > current_serial ||
-	  (current_grab->serial_end == current_serial &&
-	   current_grab->grab_one_pointer_release_event))
+      if (current_grab->serial_end > current_serial)
 	{
 	  /* This one hasn't ended yet.
 	     its the currently active one or scheduled to be active */
