@@ -47,6 +47,7 @@
 *tp++ = gdk_pixbuf_animation_get_type();
 *tp++ = gdk_pixbuf_animation_iter_get_type();
 *tp++ = gdk_pixbuf_error_get_type();
+*tp++ = gdk_pixbuf_format_get_type();
 *tp++ = gdk_pixbuf_get_type();
 *tp++ = gdk_pixbuf_loader_get_type();
 *tp++ = gdk_pixbuf_rotation_get_type();
@@ -118,6 +119,7 @@
 *tp++ = gtk_cell_renderer_pixbuf_get_type();
 *tp++ = gtk_cell_renderer_progress_get_type();
 *tp++ = gtk_cell_renderer_spin_get_type();
+*tp++ = gtk_cell_renderer_spinner_get_type();
 *tp++ = gtk_cell_renderer_state_get_type();
 *tp++ = gtk_cell_renderer_text_get_type();
 *tp++ = gtk_cell_renderer_toggle_get_type();
@@ -138,7 +140,6 @@
 *tp++ = gtk_ctree_expansion_type_get_type();
 *tp++ = gtk_ctree_line_style_get_type();
 *tp++ = gtk_ctree_pos_get_type();
-*tp++ = gtk_curve_get_type();
 *tp++ = gtk_curve_type_get_type();
 *tp++ = gtk_debug_flag_get_type();
 *tp++ = gtk_delete_type_get_type();
@@ -170,7 +171,6 @@
 *tp++ = gtk_font_selection_dialog_get_type();
 *tp++ = gtk_font_selection_get_type();
 *tp++ = gtk_frame_get_type();
-*tp++ = gtk_gamma_curve_get_type();
 *tp++ = gtk_handle_box_get_type();
 *tp++ = gtk_hbox_get_type();
 *tp++ = gtk_hbutton_box_get_type();
@@ -200,7 +200,6 @@
 *tp++ = gtk_im_preedit_style_get_type();
 *tp++ = gtk_im_status_style_get_type();
 *tp++ = gtk_info_bar_get_type();
-*tp++ = gtk_input_dialog_get_type();
 *tp++ = gtk_invisible_get_type();
 *tp++ = gtk_item_get_type();
 *tp++ = gtk_justification_get_type();
@@ -226,6 +225,7 @@
 *tp++ = gtk_number_up_layout_get_type();
 *tp++ = gtk_object_flags_get_type();
 *tp++ = gtk_object_get_type();
+*tp++ = gtk_offscreen_window_get_type();
 *tp++ = gtk_orientable_get_type();
 *tp++ = gtk_orientation_get_type();
 *tp++ = gtk_pack_direction_get_type();
@@ -303,6 +303,7 @@
 *tp++ = gtk_sort_type_get_type();
 *tp++ = gtk_spin_button_get_type();
 *tp++ = gtk_spin_button_update_policy_get_type();
+*tp++ = gtk_spinner_get_type();
 *tp++ = gtk_spin_type_get_type();
 *tp++ = gtk_state_type_get_type();
 *tp++ = gtk_statusbar_get_type();
@@ -336,6 +337,9 @@
 *tp++ = gtk_toolbar_style_get_type();
 *tp++ = gtk_tool_button_get_type();
 *tp++ = gtk_tool_item_get_type();
+*tp++ = gtk_tool_item_group_get_type();
+*tp++ = gtk_tool_palette_drag_targets_get_type();
+*tp++ = gtk_tool_palette_get_type();
 *tp++ = gtk_tool_shell_get_type();
 *tp++ = gtk_tooltip_get_type();
 *tp++ = gtk_tree_drag_dest_get_type();
